#include <qlcdnumber.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qhbox.h>
#include <qslider.h>

#include "handmade.h"

HandMade::HandMade( QWidget *parent, char *name ) : QVBox( parent, name )
{
	// A horizontal box for the list, slider and LCD
	QHBox *hb = new QHBox( this );

	// The list is placed first (i.e. to the left)
	lb = new QListBox( hb );

	// A vertical box to put the LCD above the slider
	// This is placed to the right of the list
	QVBox *vb = new QVBox( hb );

	// The LCD is created first and is thus shown at the top
	lcd = new QLCDNumber( vb );
	// The slider is created second and is this placed below the LCD
	QSlider *s = new QSlider( Horizontal, vb );
	// We connect the slider valueChanged(int) signal to the LCD
	connect( s, SIGNAL(valueChanged(int)), lcd, SLOT(display(int)) );

	// A horizontal box for the buttons, below the list, slider and LCD
	hb = new QHBox( this );

	// The add button will be to the left
	QPushButton *pb = new QPushButton( "Add", hb );
	// Connect it to the addNumber() slot
	connect( pb, SIGNAL(clicked()), this, SLOT(addNumber()) );
	// The remove button with be to the right of the add button
	pb = new QPushButton( "Remove", hb );
	// Connect it to the removeNumber slot
	connect( pb, SIGNAL(clicked()), this, SLOT(removeNumber()) );
}

void HandMade::addNumber()
{
	// Simply insert the number shown by the LCD
	lb->insertItem( QString::number( lcd->intValue() ) );
}

void HandMade::removeNumber()
{
	// Return if no item is selected
	if( lb->currentItem() < 0 )
		return;

	// Remove the selected item
	lb->removeItem( lb->currentItem() );
}
