/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename slots use Qt Designer which will
** update this file, preserving your code. Create an init() slot in place of
** a constructor, and a destroy() slot in place of a destructor.
*****************************************************************************/


void frmMain::addContact()
{
  // Create a new dialog
  dlgContact dlg( this );
    
  // Show it and wait for Ok or Cancel
  if( dlg.exec() == QDialog::Accepted )
  {
	  // We got Ok
	  Contact c;
	
	  // Extract the information from the dialog
	  c.name = dlg.leName->text();
	  c.eMail = dlg.leEMail->text();
	  c.phone = dlg.lePhone->text();
	
		// Check for duplicate names
		for( QValueList<Contact>::iterator it = m_contacts.begin(); it != m_contacts.end(); ++it )
		{
			if( (*it).name == c.name )
	    {
        // Warn for duplicates and fail
    		QMessageBox::warning( this, "Duplicate", "The person " + (*it).name + " allready exists." );
    		return;
	    }
		}
		// No duplicates found, add the contact to the list and show it in the listview
		m_contacts.append( c );
		lvContacts->insertItem( new QListViewItem( lvContacts, c.name , c.eMail, c.phone ) );
  }
}

void frmMain::editContact()
{
  // Get a pointer to the selected item
  QListViewItem *item = lvContacts->currentItem();
    
    // Check if there is a valid select item
  if( item )
  {
		// Find the corresponding item in the internal list
		for( QValueList<Contact>::iterator it = m_contacts.begin(); it != m_contacts.end(); ++it )
		{
	    if( (*it).name == item->text(0) )
	    {
				// We have found it, show the dialog with it
				dlgContact dlg( this );
		
				dlg.leName->setText( (*it).name );
				dlg.leEMail->setText( (*it).eMail );
				dlg.lePhone->setText( (*it).phone );
		
				// Show it and wait for Ok or Cancel
				if( dlg.exec() == QDialog::Accepted )
				{
					// Check for duplicate names by counting the number of occurencies
					int occurencies = 0;
					for( QValueList<Contact>::iterator i2 = m_contacts.begin(); i2 != m_contacts.end(); ++i2 )		   
						if( (*i2).name == dlg.leName->text() )
							occurencies++;
		 
					// If the name is changed we allow no occurencies, otherwise one 
					if( (dlg.leName->text() != (*it).name && occurencies > 0) || (occurencies > 1) )
					{
						// Warn for duplicates and fail
						QMessageBox::warning( this, "Duplicate", "The person " + dlg.leName->text() + " allready exists." );
						return;
					}
		    
					// Update the internal list
					(*it).name = dlg.leName->text();
					(*it).eMail = dlg.leEMail->text();
					(*it).phone = dlg.lePhone->text();
		    
					// Update the list view
					item->setText( 0, (*it).name );
					item->setText( 1, (*it).eMail );
					item->setText( 2, (*it).phone );
				}

				return;
			}
		}
	}
}

void frmMain::init()
{
  // Clear the list
  lvContacts->clear();
}

void frmMain::removeContact()
{
  // Get a pointer to the selected item
  QListViewItem *item = lvContacts->currentItem();
    
  // Check if there is a valid select item
  if( item )
  {
		// Find the corresponding item in the internal list
		for( QValueList<Contact>::iterator it = m_contacts.begin(); it != m_contacts.end(); ++it )
		{
	    if( (*it).name == item->text(0) )
	    {
				// We have found it, remove it from the list and listview
				m_contacts.remove( it );
				delete item;
				return;
	    }
		}
  }
}


void frmMain::loadFile()
{
  QString filename = QFileDialog::getOpenFileName( QString::null, "Addressbooks (*.adb)", this, "file open", "Addressbook File Open" );
    
  if ( !filename.isEmpty() )
  {
		m_filename = filename;
		load( filename );
	}
}

void frmMain::saveFile()
{
	if( m_filename.isEmpty() )
	{
		saveFileAs();
		return;
	}
    
	save( m_filename );
}

void frmMain::saveFileAs()
{
	QString filename = QFileDialog::getSaveFileName( QString::null, "Addressbooks (*.adb)", this, "file save as", "Addressbook Save As" );    
	if ( !filename.isEmpty() )
	{
		m_filename = filename;
		save( m_filename );
	}
}

void frmMain::load( const QString &filename )
{
	QFile file( filename );
    
	if( !file.open( IO_ReadOnly ) )
	{
		QMessageBox::warning( this, "Loading", "Failed to load file." );
		return;
	}
 
	QDomDocument doc( "AdBookML" );
	if( !doc.setContent( &file ) )
	{
		QMessageBox::warning( this, "Loading", "Failed to load file." );
		file.close();
		return;
	}
	
	file.close();
	
	QDomElement root = doc.documentElement();
	if( root.tagName() != "adbook" )
	{
		QMessageBox::warning( this, "Loading", "Invalid file." );
		return;
	}
	
	m_contacts.clear();
	lvContacts->clear();
	
	QDomNode n = root.firstChild();
	while( !n.isNull() )
	{
		QDomElement e = n.toElement();
		if( !e.isNull() )
		{
			if( e.tagName() == "contact" )
			{
				Contact c( e );
			
				m_contacts.append( c );
				lvContacts->insertItem( new QListViewItem( lvContacts, c.name , c.eMail, c.phone ) );
			}
		}
		
		n = n.nextSibling();
	}
}

void frmMain::save( const QString &filename )
{
	QDomDocument doc( "AdBookML" );
	QDomElement root = doc.createElement( "adbook" );
	doc.appendChild( root );
	
	for( QValueList<Contact>::iterator it = m_contacts.begin(); it != m_contacts.end(); ++it )
		root.appendChild( (*it).createXMLNode( doc ) );
	
	QFile file( filename );
	if( !file.open( IO_WriteOnly ) )
	{
		QMessageBox::warning( this, "Saving", "Failed to save file." );
		return;
	}
	
	QTextStream ts( &file );
    
	ts << doc.toString();
	
	file.close();
}
