#include <qapplication.h>
#include <qpainter.h>
#include <qcanvas.h>

class MyCanvasView : public QCanvasView
{
public:
	MyCanvasView( QCanvas *c, QWidget *parent=0, const char *name=0, WFlags f=0 ) : QCanvasView( c, parent, name, f )
	{
		dragging = 0;
	}
	
protected:
	void contentsMousePressEvent( QMouseEvent *e )
	{
		QCanvasItemList il = canvas()->collisions( e->pos() );
		for( QCanvasItemList::Iterator it=il.begin(); it!=il.end(); ++it ) 
		{
			if( (*it)->rtti() != QCanvasText::RTTI )
			{
				dragging = (*it);
			
				xoffset = (int)(e->x() - dragging->x());
				yoffset = (int)(e->y() - dragging->y());
				
				return;
			}
		}
	}
	
	void contentsMouseReleaseEvent( QMouseEvent *e )
	{
		if( dragging )
		{
			dragging->setX( e->x() - xoffset );
			dragging->setY( e->y() - yoffset );

			dragging = 0;
			
			canvas()->update();
		}
	}
	
	void contentsMouseMoveEvent( QMouseEvent *e )
	{
		if( dragging )
		{
			dragging->setX( e->x() - xoffset );
			dragging->setY( e->y() - yoffset );

			canvas()->update();
		}
	}
	
private:
	QCanvasItem *dragging;
	int xoffset, yoffset;
};

class MyCanvasItem : public QCanvasPolygonalItem
{
public:
	MyCanvasItem( int width, int height, QCanvas *c ) : QCanvasPolygonalItem( c )
	{
		m_width = width;
		m_height = height;
	}
	
	~MyCanvasItem()
	{
		hide(); // Required to avoid segfault - see docs
	}
	
	QPointArray areaPoints() const
	{
		QPointArray res(8);
		
		res.setPoint( 0, QPoint( (int)x() + m_width/4,   (int)y() ) );
		res.setPoint( 1, QPoint( (int)x() + 3*m_width/4, (int)y() ) );
		res.setPoint( 2, QPoint( (int)x() + m_width,     (int)y() + 3*m_height/8 ) );
		res.setPoint( 3, QPoint( (int)x() + m_width,     (int)y() + 5*m_height/8 ) );
		res.setPoint( 4, QPoint( (int)x() + 3*m_width/4, (int)y() + m_height ) );
		res.setPoint( 5, QPoint( (int)x() + m_width/4,   (int)y() + m_height ) );
		res.setPoint( 6, QPoint( (int)x(),               (int)y() + 5*m_height/8 ) );
		res.setPoint( 7, QPoint( (int)x(),               (int)y() + 3*m_height/8 ) );
		
		return res;
	}
	
protected:
	void drawShape( QPainter &p )
	{
		p.drawEllipse( (int)x()+m_width/4, (int)y(), m_width/2, m_height );
		p.drawRect( (int)x(), (int)y()+3*m_height/8, m_width, m_height/4 );
	}
	
private:
	int m_width;
	int m_height;
};

int main( int argc, char **argv )
{
	QApplication a( argc, argv );
	
	QCanvas *c = new QCanvas( 260, 75 );
	QCanvasView *cv = new MyCanvasView( c );

	QCanvasRectangle *rect = new QCanvasRectangle( 10, 10, 40, 40, c );
	rect->setPen( Qt::black );
	rect->setBrush( Qt::red );
	rect->show();

	QCanvasText *t = new QCanvasText( "Rect", c );
	t->setX( 30 );
	t->setY( 55 );
	t->setTextFlags( Qt::AlignHCenter );
	t->show();
	
	QPointArray points( 3 );
	points.setPoint( 0, 20, 0 );
	points.setPoint( 1, 0, 40 );
	points.setPoint( 2, 40, 40 );

	QCanvasPolygon *poly = new QCanvasPolygon( c );
	poly->setPoints( points );
	poly->setX( 60 );
	poly->setY( 10 );
	poly->setBrush( Qt::blue );
	poly->show();
	
	t = new QCanvasText( "Poly", c );
	t->setX( 80 );
	t->setY( 55 );
	t->setTextFlags( Qt::AlignHCenter );
	t->show();
	
	QCanvasLine *line = new QCanvasLine( c );
	line->setPoints( 110, 10, 150, 50 );
	line->setPen( QPen( Qt::green, 4 ) );
	line->show();
	
	t = new QCanvasText( "Line", c );
	t->setX( 130 );
	t->setY( 55 );
	t->setTextFlags( Qt::AlignHCenter );
	t->show();
	
	QCanvasEllipse *elli = new QCanvasEllipse( 40, 40, 45*16, 225*16, c );
	elli->setX( 180 );
	elli->setY( 30 );
	elli->setBrush( Qt::cyan );
	elli->show();
	
	t = new QCanvasText( "Elli", c );
	t->setX( 180 );
	t->setY( 55 );
	t->setTextFlags( Qt::AlignHCenter );
	t->show();
	
	MyCanvasItem *my = new MyCanvasItem( 40, 40, c );
	my->setX( 210 );
	my->setY( 10 );
	my->setBrush( Qt::yellow );
	my->setPen( Qt::black );
	my->show();

	t = new QCanvasText( "Custom", c );
	t->setX( 230 );
	t->setY( 55 );
	t->setTextFlags( Qt::AlignHCenter );
	t->show();
		
	c->update();

	a.setMainWidget( cv );
	cv->show();
	
	return a.exec();
}
