#include "listviewexample.h"

#include <qlistview.h>
#include <qlabel.h>
#include <qpixmap.h>

class SortItem : public QListViewItem
{
public:
	SortItem( QListView *parent, QString c1, QString c2, QString c3, QString c4 ) : QListViewItem( parent, c1, c2, c3, c4 )
	{
	}
	
	int compare( QListViewItem *i, int col, bool asc ) const
	{
		if( col == 2 )
			return text( col ).toInt() - i->text( col ).toInt();
		else
			return QListViewItem::compare( i, col, asc );
	}
};

ListViewExample::ListViewExample( QWidget *parent, char *name ) : QTabWidget( parent, name )
{
	addTab( setupListTab(), "List" );
	addTab( setupTreeTab(), "Tree" );
	addTab( setupSortTab(), "Sorting" );
	addTab( setupIconTab(), "Icons" );
}

QWidget *ListViewExample::setupListTab()
{
	m_listView = new QListView();
	
	m_listView->addColumn( "Foo" );
	m_listView->addColumn( "Bar" );
	m_listView->addColumn( "Baz" );
	
	m_listView->setAllColumnsShowFocus( true );
	
	new QListViewItem( m_listView, "(1, 1)", "(1, 2)", "(1, 3)" );
	new QListViewItem( m_listView, "(2, 1)", "(2, 2)", "(2, 3)" );
	new QListViewItem( m_listView, "(3, 1)", "(3, 2)", "(3, 3)" );
	new QListViewItem( m_listView, "(4, 1)", "(4, 2)", "(4, 3)" );
	
	return m_listView;
}

QWidget *ListViewExample::setupTreeTab()
{
	m_treeView = new QListView();
	
	m_treeView->addColumn( "Tree" );
	m_treeView->addColumn( "First" );
	m_treeView->addColumn( "Second" );
	m_treeView->addColumn( "Third" );
	
	m_treeView->setRootIsDecorated( true );

	QListViewItem *root = new QListViewItem( m_treeView, "root" );
	
	QListViewItem *a = new QListViewItem( root, "A" );
	QListViewItem *b = new QListViewItem( root, "B" );
	QListViewItem *c = new QListViewItem( root, "C" );
	
	new QListViewItem( a, "foo", "1", "2", "3" );
	new QListViewItem( a, "bar", "i", "ii", "iii" );
	new QListViewItem( a, "baz", "a", "b", "c" );

	new QListViewItem( b, "foo", "1", "2", "3" );
	new QListViewItem( b, "bar", "i", "ii", "iii" );
	new QListViewItem( b, "baz", "a", "b", "c" );

	new QListViewItem( c, "foo", "1", "2", "3" );
	new QListViewItem( c, "bar", "i", "ii", "iii" );
	new QListViewItem( c, "baz", "a", "b", "c" );
	
	return m_treeView;
}

QWidget *ListViewExample::setupSortTab()
{
	m_sortView = new QListView();
	
	m_sortView->addColumn( "Number" );
	m_sortView->addColumn( "Padded" );
	m_sortView->addColumn( "Corrected" );
	m_sortView->addColumn( "Alphabetical" );
	
	m_sortView->setShowSortIndicator( true );
	
	new SortItem( m_sortView, "1", "02", "1", "foo" );
	new SortItem( m_sortView, "3", "04", "1", "bar" );
	new SortItem( m_sortView, "5", "06", "2", "foo" );
	new SortItem( m_sortView, "7", "08", "3", "bar" );
	new SortItem( m_sortView, "9", "10", "5", "Foo" );
	new SortItem( m_sortView, "11", "12", "8", "bar" );
	new SortItem( m_sortView, "13", "14", "13", "foo" );
	new SortItem( m_sortView, "15", "00", "21", "Bar" );
	
	return m_sortView;
}

QWidget *ListViewExample::setupIconTab()
{
	m_iconView = new QListView();

	m_iconView->addColumn( "Foo" );
	m_iconView->addColumn( "Bar" );
	m_iconView->addColumn( "Baz" );
	
	m_iconView->setAllColumnsShowFocus( true );
	
	QPixmap pm( 10, 10 );
	
	QListViewItem *lvi = new QListViewItem( m_iconView, "(1, 1)", "(1, 2)", "(1, 3)" );
	pm.fill( Qt::red );
	lvi->setPixmap( 0, pm );
	
	lvi = new QListViewItem( m_iconView, "(2, 1)", "(2, 2)", "(2, 3)" );
	pm.fill( Qt::green );
	lvi->setPixmap( 1, pm );

	lvi = new QListViewItem( m_iconView, "(3, 1)", "(3, 2)", "(3, 3)" );
	pm.fill( Qt::blue );
	lvi->setPixmap( 2, pm );

	lvi = new QListViewItem( m_iconView, "(4, 1)", "(4, 2)", "(4, 3)" );	
	pm.fill( Qt::yellow );
	lvi->setPixmap( 0, pm );
	pm.fill( Qt::magenta );
	lvi->setPixmap( 2, pm );
	
	return m_iconView;
}

